@@ -1,14 +1,13 @@
-
 package Number::Bytes::Human;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.07';
+our $VERSION = '0.10';
 
 require Exporter;
 our @ISA = qw(Exporter);
-our @EXPORT_OK = qw(format_bytes);
+our @EXPORT_OK = qw(format_bytes parse_bytes);
 
 require POSIX;
 use Carp qw(croak carp);
@@ -35,11 +34,10 @@ sub _default_suffixes {
 }
 
 my %ROUND_FUNCTIONS = (
-  ceil => \&POSIX::ceil,
-  floor => \&POSIX::floor,
-  #round => sub { shift }, # FIXME
-  #trunc => sub { int shift } # FIXME
-
+  ceil => sub { return POSIX::ceil($_[0] * (10 ** $_[1])) / 10**$_[1]; },
+  floor => sub { return POSIX::floor($_[0] * (10 ** $_[1])) / 10**$_[1]; },
+  round => sub { return sprintf( "%." . ( $_[1] || 0 ) . "f", $_[0] ); },
+  trunc => sub { return sprintf( "%d", $_[0] * (10 ** $_[1])) / 10**$_[1]; },
   # what about 'ceiling'?
 );
 
@@ -65,7 +63,7 @@ sub _round_function {
 #
 #   zero => '0' (default) | '-' | '0%S' | undef
 #
-#   
+#
 #   supress_point_zero | no_point_zero =>
 #   b_to_i => 1
 #   to_s => \&
@@ -77,17 +75,18 @@ sub _round_function {
 
 
 #  PROBABLY CRAP:
-#   precision =>
+#   precision => integer
 
 # parsed options
-#   BLOCK => 1024 | 1020
+#   BLOCK => 1024 | 1000
 #   ROUND_STYLE => 'ceil', 'round', 'floor', 'trunc'
 #   ROUND_FUNCTION => \&
 #   SUFFIXES => \@
 #   ZERO =>
+#   SI => undef | 1			Parse SI compatible
 
 
-=begin private 
+=begin private
 
   $options = _parse_args($seed, $args)
   $options = _parse_args($seed, arg1 => $val1, ...)
@@ -109,8 +108,12 @@ sub _parse_args {
     $options{ROUND_STYLE} = 'ceil';
     $options{ROUND_FUNCTION} = _round_function($options{ROUND_STYLE});
     $options{ZERO} = '0';
+    $options{SI} = undef;
+    $options{PRECISION} = 1;
+    $options{PRECISION_CUTOFF} = 1;
     #$options{SUFFIXES} = # deferred to the last minute when we know BLOCK, seek [**]
-  } 
+    $options{UNIT} = undef;
+  }
   # else { %options = %$seed } # this is set if @_!=0, down below
 
   if (@_==0) { # quick return for default values (no customized args)
@@ -122,7 +125,7 @@ sub _parse_args {
   }
 
   # this is done here so this assignment/copy doesn't happen if @_==0
-  %options = %$seed unless %options; 
+  %options = %$seed unless %options;
 
 # block | block_size | base | bs => 1024 | 1000
 # block_1024 | base_1024 | 1024 => $true
@@ -140,7 +143,7 @@ sub _parse_args {
       croak "invalid base: $block (should be 1024, 1000 or 1024000)";
     }
     $options{BLOCK} = $block;
-    
+
   } elsif ($args{block_1024} ||
            $args{base_1024}  ||
            $args{1024}) {
@@ -166,6 +169,11 @@ sub _parse_args {
     $options{ROUND_STYLE} = $args{round_style};
   }
 
+# SI compatibility (mostly for parsing)
+  if ($args{si}) {
+    $options{SI} = 1;
+  }
+
 # suffixes => 1024 | 1000 | si_1024 | si_1000 | 1024000 | \@
   if ($args{suffixes}) {
     if (ref $args{suffixes} eq 'ARRAY') {
@@ -175,22 +183,29 @@ sub _parse_args {
     } else {
       croak "suffixes ($args{suffixes}) should be 1024, 1000, si_1024, si_1000, 1024000 or an array ref";
     }
-  } elsif ($args{si}) {
-    my $set = ($options{BLOCK}==1024) ? 'si_1024' : 'si_1000';
-    $options{SUFFIXES} = _default_suffixes($set);
-  } elsif (defined $args{unit}) {
-    my $suff = $args{unit};
-    $options{SUFFIXES} = [ map  { "$_$suff" } @DEFAULT_PREFIXES ];
+  }
+  if (defined $args{unit}) {
+    $options{UNIT} = $args{unit};
   }
 
 # zero => undef | string
   if (exists $args{zero}) {
     $options{ZERO} = $args{zero};
     if (defined $options{ZERO}) {
-      $options{ZERO} =~ s/%S/$options{SUFFIXES}->[0]/g 
+      $options{ZERO} =~ s/%S/$options{SUFFIXES}->[0]/g
     }
   }
 
+# precision => <integer>
+  if (exists $args{precision} and $args{precision} =~ /\A\d+\z/) {
+    $options{PRECISION} = $args{precision};
+  }
+
+# precision_cutoff => <intenger>
+  if (exists $args{precision_cutoff} and ($args{precision_cutoff} =~ /\A\d+\z/ or $args{precision_cutoff} = '-1')) {
+    $options{PRECISION_CUTOFF} = $args{precision_cutoff};
+  }
+
 # quiet => 1
   if ($args{quiet}) {
     $options{QUIET} = 1;
@@ -218,7 +233,7 @@ sub _format_bytes {
   my $block = $options{BLOCK};
 
   # if a suffix set was not specified, pick a default [**]
-  my @suffixes = $options{SUFFIXES} ? @{$options{SUFFIXES}} : _default_suffixes($block);
+  my @suffixes = $options{SUFFIXES} ? @{$options{SUFFIXES}} : _default_suffixes( ($options{SI} ? 'si_' : '') . $block);
 
   # WHAT ABOUT NEGATIVE NUMBERS: -1K ?
   my $sign = '';
@@ -226,37 +241,128 @@ sub _format_bytes {
      $bytes = -$bytes;
      $sign = '-';
   }
-  return $sign . human_round($bytes) . $suffixes[0] if $bytes<$block;
-
-#  return "$sign$bytes" if $bytes<$block;
 
+  my $suffix = $suffixes[0];
   my $x = $bytes;
-  my $suffix;
-  foreach (@suffixes) {
-    $suffix = $_, last if human_round($x) < $block;
+  my $magnitude = 0;
+  if($bytes >= $block) {
+  #  return "$sign$bytes" if $bytes<$block;
+    do {
+      $x /= $block;
+      $magnitude++;
+    } while ( human_round($x, $options{PRECISION}) >= $block );
+    if($magnitude >= (0 + @suffixes)) {
+      carp "number too large (>= $block**$magnitude)" unless ($options{QUIET});
+    }
+    $suffix = $suffixes[$magnitude];
+  }
+  #$x = human_round( $x, $options{PRECISION} );
+
+  $x = _precision_cutoff($x, $options);
+  #reasses encase the precision_cutoff caused the value to cross the block size
+  if($x >= $block) {
     $x /= $block;
-  }
-  unless (defined $suffix) { # number >= $block*($block**@suffixes) [>= 1E30, that's huge!]
-      unless ($options{QUIET}) {
-        my $pow = @suffixes+1; 
-        carp "number too large (>= $block**$pow)"
-      }
-      $suffix = $suffixes[-1];
-      $x *= $block;
-  }
-  # OPTION: return "Inf"
-
-  my $num;
-  if ($x < 10.0) {
-    $num = sprintf("%.1f", human_round($x*10)/10); 
-  } else {
-    $num = sprintf("%d", human_round($x));
+    $magnitude++;
+    if($magnitude >= (0 + @suffixes)) {
+      carp "number too large (>= $block**$magnitude)" unless ($options{QUIET});
+    }
+    $suffix = $suffixes[$magnitude];
+    $x = _precision_cutoff($x, $options);
   }
 
-  "$sign$num$suffix"
+  my $unit = $options{UNIT} || '';
+
+  return $sign . $x . $suffix . $unit;
 
 }
 
+sub _precision_cutoff {
+ my $bytes   = shift;
+ my $options = shift;
+ my %options = %$options;
+ if ( $options{PRECISION_CUTOFF} != -1 and ( length( sprintf( "%d", $bytes ) ) > $options{PRECISION_CUTOFF} ) ) {
+   $bytes = sprintf( "%d", human_round( $bytes, 0 ) );
+ } else {
+   $bytes = sprintf( "%." . $options{PRECISION} . "f", human_round( $bytes, $options{PRECISION} ) );
+ }
+ return $bytes;
+}
+
+sub _parse_bytes {
+  my $human = shift;
+  my $options = shift;
+  my %options = %$options;
+
+  return 0 if( exists $options{ZERO} && ((!defined $options{ZERO} && !defined $human) || (defined $human && $human eq $options{ZERO})) );
+  return undef unless defined $human;
+
+  my %suffix_mult;
+  my %suffix_block;
+  my $m;
+
+  if( $options{SUFFIXES} ) {
+    $m = 1;
+    foreach my $s (@{$options{SUFFIXES}}) {
+      $suffix_mult{$s} = $m;
+      $suffix_block{$s} = $options{BLOCK};
+      $m *= $suffix_block{$s};
+    }
+  } else {
+    if( !defined $options{SI} || $options{SI} == 1 ) {
+      # If SI compatibility has been set BLOCK is ignored as it is infered from the unit
+      $m = 1;
+      foreach my $s (@{$DEFAULT_SUFFIXES{si_1000}}) {
+        $suffix_mult{$s} = $m;
+        $suffix_block{$s} = 1000;
+        $m *= $suffix_block{$s};
+      }
+    
+      $m = 1;
+      foreach my $s (@{$DEFAULT_SUFFIXES{si_1024}}) {
+        $suffix_mult{$s} = $m;
+        $suffix_block{$s} = 1024;
+        $m *= $suffix_block{$s};
+      }
+    }
+
+    # The regular suffixes are only taken into account in default mode without specifically asking for SI compliance
+    if( !defined $options{SI} ) {
+      $m = 1;
+      foreach my $s (_default_suffixes( $options{BLOCK} )) {
+        $suffix_mult{$s} = $m;
+        $suffix_block{$s} = $options{BLOCK};
+        $m *= $suffix_block{$s};
+      }
+    }
+  }
+
+  my ($sign, $k, $unit) = ($human =~ /^\s*(-?)\s*(\d*(?:\.\d*)?)\s*(\D*)$/);
+
+#  print STDERR "S: $sign K: $k U: $unit\n";
+
+
+  my $mult;
+  my $u = $options{UNIT} || '';
+  foreach my $s (keys %suffix_block) {
+    if( $unit =~ /^${s}${u}$/i ) {
+      $mult = ($sign eq '-' ? -1 : 1) * $suffix_mult{$s};
+      last;
+    }
+  }
+
+  if( !defined $mult ) {
+    carp "Could not parse human readable byte value '$human'";
+use Data::Dumper;
+print STDERR Dumper( %suffix_block );
+    return undef;
+  }
+
+  my $bytes = int( $k * $mult );
+
+  return $bytes;
+}
+
+
 # convert byte count (file size) to human readable format
 sub format_bytes {
   my $bytes = shift;
@@ -265,6 +371,14 @@ sub format_bytes {
   return _format_bytes($bytes, $options);
 }
 
+# convert human readable format to byte count (file size)
+sub parse_bytes {
+  my $human = shift;
+  my $options = _parse_args(undef, @_);
+  #use YAML; print Dump $options;
+  return _parse_bytes($human, $options);
+}
+
 ### the OO way
 
 # new()
@@ -288,8 +402,14 @@ sub format {
   return _format_bytes($bytes, $self);
 }
 
+# parse()
+sub parse {
+  my $self = shift;
+  my $human = shift;
+  return _parse_bytes($human, $self);
+}
 
-# the solution by COG in Filesys::DiskUsage 
+# the solution by COG in Filesys::DiskUsage
 # convert size to human readable format
 #sub _convert {
 #  defined (my $size = shift) || return undef;
@@ -323,18 +443,31 @@ Number::Bytes::Human - Convert byte count to human readable format
 
 =head1 SYNOPSIS
 
-  use Number::Bytes::Human qw(format_bytes);
+  use Number::Bytes::Human qw(format_bytes parse_bytes);
   $size = format_bytes(0); # '0'
   $size = format_bytes(2*1024); # '2.0K'
 
   $size = format_bytes(1_234_890, bs => 1000); # '1.3M'
   $size = format_bytes(1E9, bs => 1000); # '1.0G'
 
+  my $bytes = parse_bytes('1.0K');   # 1024
+  my $bytes = parse_bytes('1.0KB');  # 1000, SI unit
+  my $bytes = parse_bytes('1.0KiB'); # 1024, SI unit
+
   # the OO way
   $human = Number::Bytes::Human->new(bs => 1000, si => 1);
   $size = $human->format(1E7); # '10MB'
+
+  $bytes = $human->parse('10MB');   # 10*1000*1000
+  $bytes = $human->parse('10MiB');  # 10*1024*1024
+  $bytes = $human->parse('10M');    # Error, no SI unit
+
   $human->set_options(zero => '-');
-  $size = $human->format(0); # '-'
+  $size = $human->format(0);    # '-'
+  $bytes = $human->parse('-');  # 0
+
+  $human = Number::Bytes::Human->new(bs => 1000, round_style => 'round', precision => 2);
+  $size = $human->format(10240000); # '10.24MB'
 
 =head1 DESCRIPTION
 
@@ -398,12 +531,19 @@ refactored, abused. And then, when it is much improved, some
 brave soul can port it back to C (if only for the warm feeling
 of painful programming).
 
+It is also possible to parse human readable formatted bytes. The 
+automatic format detection recognizes SI units with the blocksizes
+of 1000 and 1024 respectively and additionally the customary K / M / G etc. with
+blocksize 1024. When si => 1 is added to the options only SI units
+are recognized. Explicitly specifying a blocksize changes it
+for all detected units.
+
 =head2 OBJECTS
 
 An alternative to the functional style of this module
-is the OO fashion. This is useful for avoiding the 
+is the OO fashion. This is useful for avoiding the
 unnecessary parsing of the arguments over and over
-if you have to format lots of numbers 
+if you have to format lots of numbers
 
 
   for (@sizes) {
@@ -435,6 +575,12 @@ Turns a byte count (like 1230) to a readable format like '1.3K'.
 You have a bunch of options to play with. See the section
 L</"OPTIONS"> to know the details.
 
+=item B<parse_bytes>
+
+  $size = parse_bytes($h_size, @options);
+
+Turns a human readable byte count into a number of the equivalent bytes.
+
 =back
 
 =head2 METHODS
@@ -453,7 +599,7 @@ L</"OPTIONS">.
   $h_size = $h->format($size);
 
 Turns a byte count (like 1230) to a readable format like '1.3K'.
-The statements 
+The statements
 
   $h = Number::Bytes::Human->new(@options);
   $h_size = $h->format($size);
@@ -461,6 +607,19 @@ The statements
 are equivalent to C<$h_size = format_bytes($size, @options)>,
 with only one pass for the option arguments.
 
+=item B<parse>
+
+  $size = $h->parse($h_size)
+
+Turns a human readable byte count into the number of bytes.
+The statements
+
+  $h = Number::Bytes::Human->new(@options);
+  $size = $h->format($h_size);
+
+are equivalent to C<$size = parse_bytes($h_size, @options)>,
+with only one pass for the option arguments.
+
 =item B<set_options>
 
   $h->set_options(@options);
@@ -472,7 +631,7 @@ See L</"OPTIONS">.
 
 =head2 OPTIONS
 
-=over 4 
+=over 4
 
 =item BASE
 
@@ -486,9 +645,9 @@ Any other value throws an exception.
 
 =item SUFFIXES
 
-  suffixes => 1000 | 1024 | 1024000 | si_1000 | si_1024 | $arrayref 
+  suffixes => 1000 | 1024 | 1024000 | si_1000 | si_1024 | $arrayref
 
-By default, the used suffixes stand for '', 'K', 'M', ... 
+By default, the used suffixes stand for '', 'K', 'M', ...
 for base 1024 and '', 'k', 'M', ... for base 1000
 (which are indeed the usual metric prefixes with implied unit
 as bytes, 'B'). For the weird 1024000 base, suffixes are
@@ -510,7 +669,7 @@ The string may contain '%S' in which case the suffix for byte is used.
 =item ROUND
 
   round_function => $coderef
-  round_style => 'ceil' | 'floor'
+  round_style => 'ceil' | 'floor' | 'round' | 'trunc'
 
 =item TO_S
 
@@ -521,11 +680,27 @@ The string may contain '%S' in which case the suffix for byte is used.
 Suppresses the warnings emitted. Currently, the only case is
 when the number is large than C<$base**(@suffixes+1)>.
 
+=item PRECISION
+
+  precision => <integer>
+
+default = 1
+sets the precicion of digits, only apropreacte for round_style 'round' or if you
+want to accept it in as the second parameter to your custome round_function.
+
+=item PRECISION_CUTOFF
+
+  precision_cutoff => <integer>
+
+default = 1
+when the number of digits exceeds this number causes the precision to be cutoff
+(was default behaviour in 0.07 and below)
+
 =back
 
 =head2 EXPORT
 
-It is alright to import C<format_bytes>, but nothing is exported by default.
+It is alright to import C<format_bytes> and C<parse_bytes>, but nothing is exported by default.
 
 =head1 DIAGNOSTICS
 
